/************************************************************************/
/* confxml.h - Configuration library Header				*/
/************************************************************************/
/*
 *  (c)2009,2010 Todd Shadburn
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/************************************************************************/


#ifndef _CONFXML_H_
#define _CONFXML_H_

#include <stdint.h>

#include <gxsma/gxsma_portable.h>


#ifndef ESUCCESS
#define	ESUCCESS			0
#endif
#ifndef EPARSE
#define	EPARSE				1001
#endif

#define	CONFIG_OP_LOAD_FILE		1
#define	CONFIG_OP_LOAD_MEMORY		2
#define	CONFIG_OP_SAVE_FILE		3
#define	CONFIG_OP_SAVE_MEMORY		4
#define	CONFIG_OP_ITEM_ADD		5
#define	CONFIG_OP_ITEM_REMOVE		6
#define	CONFIG_OP_ITEM_ATTR_ADD		7
#define	CONFIG_OP_ITEM_ATTR_CHANGE	8
#define	CONFIG_OP_ITEM_ATTR_REMOVE	9
#define	CONFIG_OP_ITEM_CONTENT_CHANGE	10


typedef struct	_Configuration		Configuration;
typedef struct	_ConfigSearch		ConfigSearch;
/* typedef struct	_ConfigItem		ConfigItem; */
typedef void	*ConfigItem;
typedef int (*ConfigChangeCallback) (Configuration *c,
				void *arg,
				uint32_t operation);

GXSMA_API int configuration_init (ConfigChangeCallback callback);
GXSMA_API int configuration_uninit (void);

GXSMA_API int configuration_new (Configuration **cp);
GXSMA_API int configuration_free (Configuration **cp);

GXSMA_API int configuration_load_file (Configuration *c,
				char *filepath);
GXSMA_API int configuration_load_memory (Configuration *c,
				char *buffer,
				int bufferlen);
GXSMA_API int configuration_save_file (Configuration *c,
				char *filepath);
GXSMA_API int configuration_save_memory (Configuration *c,
				char **bufferp,
				int *bufferlenp);

GXSMA_API int configuration_search_new (Configuration *c,
				ConfigSearch **csp,
				char *namespacelist);
GXSMA_API int configuration_search_begin (ConfigSearch *cs,
				char *nodepath);
GXSMA_API int configuration_search_item_count (ConfigSearch *cs);
GXSMA_API int configuration_search_next_item (ConfigSearch *cs,
				ConfigItem **cip);
GXSMA_API int configuration_search_add_item (ConfigSearch *cs,
				ConfigItem *ci);
GXSMA_API int configuration_search_remove_item (ConfigSearch *cs,
				ConfigItem *ci);
GXSMA_API int configuration_search_end (ConfigSearch *cs);
GXSMA_API int configuration_search_free (ConfigSearch **csp);

GXSMA_API int configuration_item_new (Configuration *c, ConfigItem **cip, char *name);
GXSMA_API char *configuration_item_get_attribute (ConfigItem *ci,
					char *attribute);
GXSMA_API int configuration_item_add_attribute (ConfigItem *ci,
					char *attribute,
					char *value);
GXSMA_API int configuration_item_set_attribute (ConfigItem *ci,
					char *attribute,
					char *value);
GXSMA_API int configuration_item_remove_attribute (ConfigItem *ci,
					char *attribute);
GXSMA_API char *configuration_item_get_content (ConfigItem *ci);
GXSMA_API int configuration_item_set_content (ConfigItem *ci,
					char *content);
GXSMA_API int configuration_item_add_child (ConfigItem *ci,
					ConfigItem *cichild);
GXSMA_API ConfigItem *configuration_item_get_first_child (ConfigItem * ci);
GXSMA_API ConfigItem *configuration_item_get_next_sibling (ConfigItem * ci);
GXSMA_API int configuration_set_root_item (Configuration *c,
					ConfigItem *ci);


#endif


