

#include <string.h>
#include <errno.h>
#ifndef WIN32
#include <netinet/in.h>
#else
#include <winsock2.h>
#include <windows.h>
#endif

#include <stdarg.h>
#include <stdio.h>

#include <gxsma/gxsma.h>
#include <gxsma/nmodule_p.h>
#include <gxsma/event_p.h>

/* v2 stuff */

GXSMA_API int gxsma_header_clear (GxsmaHeader *nh)
{
	if (nh == NULL)
		return EINVAL;
	memset (nh, 0, sizeof (GxsmaHeader));
	return GXSMA_RC_SUCCESS;
}


GXSMA_API int gxsma_header_set_flags (GxsmaHeader *nh, u_int32_t flags)
{
	if (nh == NULL)
		return EINVAL;
	nh->flags = flags;
	return GXSMA_RC_SUCCESS;
}


GXSMA_API int gxsma_header_get_flags (GxsmaHeader *nh, u_int32_t *pflags)
{
	if (nh == NULL || pflags == NULL)
		return EINVAL;
	*pflags = nh->flags;
	return GXSMA_RC_SUCCESS;
}


GXSMA_API int gxsma_header_set_status (GxsmaHeader *nh, u_int32_t status)
{
	if (nh == NULL)
		return EINVAL;
	nh->status = status;
	return GXSMA_RC_SUCCESS;
}


GXSMA_API int gxsma_header_get_status (GxsmaHeader *nh, u_int32_t *pstatus)
{
	if (nh == NULL || pstatus == NULL)
		return EINVAL;
	*pstatus = nh->status;
	return GXSMA_RC_SUCCESS;
}


GXSMA_API int gxsma_header_set_datalen (GxsmaHeader *nh, u_int32_t datalen)
{
	if (nh == NULL)
		return EINVAL;
	nh->datalen = datalen;
	return GXSMA_RC_SUCCESS;
}


GXSMA_API int gxsma_header_get_datalen (GxsmaHeader *nh, u_int32_t *pdatalen)
{
	if (nh == NULL || pdatalen == NULL)
		return EINVAL;
	*pdatalen = nh->datalen;
	return GXSMA_RC_SUCCESS;
}


GXSMA_API int gxsma_header_set_rc (GxsmaHeader *nh, u_int32_t rc)
{
	if (nh == NULL)
		return EINVAL;
	nh->rc = rc;
	return GXSMA_RC_SUCCESS;
}


GXSMA_API int gxsma_header_get_rc (GxsmaHeader *nh, u_int32_t *prc)
{
	if (nh == NULL || prc == NULL)
		return EINVAL;
	*prc = nh->rc;
	return GXSMA_RC_SUCCESS;
}


GXSMA_API int gxsma_header_set_command (GxsmaHeader *nh, char *command)
{
	if (nh == NULL || command == NULL)
		return EINVAL;
	memset (nh->command, 0, GXSMA_V2_COMMAND_SIZE);
	strncpy (nh->command, command, GXSMA_V2_COMMAND_SIZE);
	nh->command[GXSMA_V2_COMMAND_SIZE-1] = 0;
	return GXSMA_RC_SUCCESS;
}


GXSMA_API int gxsma_header_get_command (GxsmaHeader *nh, char *command, int len)
{
	if (nh == NULL || command == NULL || len < GXSMA_V2_COMMAND_SIZE)
		return GXSMA_RC_INVALIDARG;
	strncpy (command, nh->command, GXSMA_V2_COMMAND_SIZE);
	command[GXSMA_V2_COMMAND_SIZE-1] = 0;
	return GXSMA_RC_SUCCESS;
}


GXSMA_API int gxsma_header_set_argdata (GxsmaHeader *nh, char *argdata, int len)
{
	if (nh == NULL || argdata == NULL)
		return GXSMA_RC_INVALIDARG;
	if (len > GXSMA_HEADER_ARGDATA_SIZE)
		return GXSMA_RC_INVALIDARG;
	memcpy (nh->argdata, argdata, len);
	nh->arglen = len;
	return GXSMA_RC_SUCCESS;
}


GXSMA_API int gxsma_header_get_argdata (GxsmaHeader *nh, char *argdata, int *plen)
{
	if (nh == NULL || argdata == NULL || plen == NULL)
		return GXSMA_RC_INVALIDARG;
	if (*plen < nh->arglen) {
		*plen = 0;
		return GXSMA_RC_INVALIDARG;
	}
	memcpy (argdata, nh->argdata, nh->arglen);
	*plen = nh->arglen;
	return GXSMA_RC_SUCCESS;
}


GXSMA_API int gxsma_header_to_networkorder (GxsmaHeader *nh)
{
	if (nh == NULL)
		return EINVAL;
	nh->status = htonl (nh->status);
	nh->flags = htonl (nh->flags);
	nh->datalen = htonl (nh->datalen);
	nh->arglen = htonl (nh->arglen);
	nh->rc = htonl (nh->rc);
	return GXSMA_RC_SUCCESS;
}


GXSMA_API int gxsma_header_to_hostorder (GxsmaHeader *nh)
{
	if (nh == NULL)
		return EINVAL;
	nh->status = ntohl (nh->status);
	nh->flags = ntohl (nh->flags);
	nh->datalen = ntohl (nh->datalen);
	nh->arglen = ntohl (nh->arglen);
	nh->rc = ntohl (nh->rc);
	return GXSMA_RC_SUCCESS;
}


GXSMA_API int	vector_log_entry (ModuleInfo *mi, char *format, ...)
{
	va_list args;
	int rc;
	if (mi == NULL || format == NULL)
		return GXSMA_RC_INVALIDARG;
	if (mi->vector == NULL || mi->vector->log_entry == NULL)
		return GXSMA_RC_NOTIMPLEMENTED;
	va_start (args, format);
	rc = (*mi->vector->log_entry)(mi, format, args);
	va_end (args);
	return GXSMA_RC_SUCCESS;
}


GXSMA_API int	vector_emit_event (ModuleInfo *mi, EventData *ed)
{
	int rc;
	if (mi == NULL || ed == NULL)
		return GXSMA_RC_INVALIDARG;
	if (mi->vector == NULL || mi->vector->emit_event == NULL)
		return GXSMA_RC_NOTIMPLEMENTED;
	rc = (*mi->vector->emit_event)(mi, ed);
	return rc;
}


GXSMA_API int	vector_eventchain_open (ModuleInfo *mi, EventChain **ec)
{
	int rc;
	if (mi == NULL || ec == NULL)
		return GXSMA_RC_INVALIDARG;
	if (mi->vector == NULL || mi->vector->eventchain_open == NULL)
		return GXSMA_RC_NOTIMPLEMENTED;
	rc = (*mi->vector->eventchain_open)(mi, ec);
	return rc;
}


GXSMA_API int	vector_eventchain_close (EventChain **ec)
{
	int rc;
	EventChain *c = NULL;
	ModuleInfo *mi = NULL;
	if (ec == NULL ||*ec == NULL)
		return GXSMA_RC_INVALIDARG;
	c = *ec;
	mi = c->moduleinfo;
	if (mi == NULL)
		return GXSMA_RC_INVALIDARG;
	if (mi->vector == NULL || mi->vector->eventchain_close == NULL)
		return GXSMA_RC_NOTIMPLEMENTED;
	rc = (*mi->vector->eventchain_close)(ec);
	return rc;
}


GXSMA_API int	vector_eventchain_emit_event (EventChain *ec, EventData *ed)
{
	int rc;
	ModuleInfo *mi = NULL;
	if (ec == NULL || ec->moduleinfo == NULL || ed == NULL)
		return GXSMA_RC_INVALIDARG;
	mi = ec->moduleinfo;
	if (mi->vector == NULL || mi->vector->eventchain_emit_event == NULL)
		return GXSMA_RC_NOTIMPLEMENTED;
	rc = (*mi->vector->eventchain_emit_event)(ec, ed);
	return rc;
}


/* v2 stuff */


GXSMA_API int gxsma_get_component_filepath(char *ostr, int ostrlen, char *subdir, char *filename)
{
	int	rc;
	char	str[256];
	char	envstr[256];

#ifndef WIN32
	if (!strcmp(subdir, "etc"))
		rc = snprintf(ostr,
				ostrlen,
				"%s%s%s",
				SYSCONFDIR,
				DIR_DELIMITER,
				filename);
	else if (!strcmp(subdir, "log"))
		rc = snprintf(ostr,
				ostrlen,
				"%s%s%s",
				LOGDIR,
				DIR_DELIMITER,
				filename);
	else if (!strcmp(subdir, "scripts"))
		rc = snprintf(ostr,
				ostrlen,
				"%s%s%s%s%s",
				DATADIR,
				DIR_DELIMITER,
				subdir,
				DIR_DELIMITER,
				filename);
	else
		return GXSMA_RC_INVALIDARG;
#else
	rc = GetEnvironmentVariable("PROGRAMFILES", envstr, 256);
	if (rc == 0)
		return GXSMA_RC_INVALIDARG;
	rc = snprintf(ostr,
		ostrlen,
		"%s%s%s%s%s%s%s",
		envstr,
		DIR_DELIMITER,
		"gxsma",
		DIR_DELIMITER,
		subdir,
		DIR_DELIMITER,
		filename);
#endif

	return GXSMA_RC_SUCCESS;
}



